#!/usr/bin/env python3

import csv
import json
import logging

import linkahead as db
from caosadvancedtools.serverside import helper
from caosadvancedtools.serverside.logging import configure_server_side_logging

from bis_utils import (create_email_with_link_text,
                       get_description_row, get_email_from_username,
                       get_options_row, send_mail_with_defaults)

ERROR_PREFIX = 'Something went wrong: '
ERROR_SUFFIX = ' Please conatct <a href="mailto:biosamples@geomar.de">biosamples@geomar.de</a> if you encounter this issue.'
BIS_URL = 'https://localhost:10443/Entity/'
logger = logging.getLogger("caosadvancedtools")


def get_parser():
    par = helper.get_argument_parser()
    return par


def get_responsible_person_id(data):
    return int(data["responsible_person"])


def get_responsible_person_abbreviation(data):
    person = db.execute_query("FIND {}".format(get_responsible_person_id(data)))[0]
    return person.get_property("Abbreviation").value


def get_number_of_samples(data):
    return int(data["number_of_samples"])
# def is_samples_from_cruise(data):
#     return ("samples_from_cruise" in data and data["samples_from_cruise"] == "on")
# def has_start_and_end_time(data):
#     return ("start_and_end_time" in data and data["start_and_end_time"] == "on")
# def get_number_of_locations(data):
#     number_of_locations = int(data["number_of_locations"])
#     if number_of_locations < 1:
#         number_of_locations = 1 # At least one location is OBLIGATORY
#     return number_of_locations


def get_reference_properties(data):
    return data["reference_properties"]


def get_column_names(data):
    # Make sure each header starts with BIS ID and Main User
    starting_names = ["BIS ID", "Main User"]
    # required_column_names is just a string of column names separated by
    # commas.
    other_names = data["required_column_names"].split(',') + data["column_names"]
    starting_names.extend([name.strip()
                          for name in other_names if name.strip() not in starting_names])
    return starting_names


def create_sample_entities(data):
    responsible_person_id = get_responsible_person_id(data)
    number_of_samples = get_number_of_samples(data)
    add_state = len(db.execute_query("FIND 'Sample life-cycle'"))

    batch = db.Container()
    for index in range(number_of_samples):
        sample = db.Record()
        sample.add_parent(name="Sample")
        sample.add_property(name="Main User", value=responsible_person_id)
        sample.add_property("Container")
        sample.add_property("NagoyaCase")
        if add_state:
            sample.state = db.State(name="Registered", model="Sample life-cycle")
        batch += [sample]
    batch.insert()
    return batch


def get_template_name(sample_entities):
    first_id = sample_entities[0].id
    last_id = sample_entities[-1].id
    return "sample_template_(IDs_{}_to_{}).csv".format(first_id, last_id)


def configure(args):
    global logger
    # Check whether executed locally or as an SSS depending on
    # auth_token argument.
    if hasattr(args, "auth_token") and args.auth_token:
        db.configure_connection(auth_token=args.auth_token)
        debug_file = configure_server_side_logging()
        logger = logging.getLogger("caosadvancedtools")
    else:
        # logging config for local execution
        logger = logging.getLogger("caosadvancedtools")
        logger.addHandler(logging.StreamHandler(sys.stdout))
        logger.setLevel(logging.DEBUG)
        debug_file = None


def read_input(args, log=False):
    if hasattr(args, "filename") and args.filename:
        # Read the input from the form (form.json)
        with open(args.filename, mode='r') as form_json:
            form_data = json.load(form_json)
        if log:
            with open(args.filename, mode='r') as form_json:
                logger.info("Loaded JOSN file: \n{}".format(form_json.read()))
        return form_data
    else:
        logger.error("{}register_new_samples.py was called without the JSON file in args.{}".format(
            ERROR_PREFIX, ERROR_SUFFIX))
        return None  # TODO: EXIT


def create_csv_template(template_name, samples, form_input):
    template_display_path, template_internal_path = helper.get_shared_filename(template_name)
    # Read form_input
    responsible_person_abbreviation = get_responsible_person_abbreviation(form_input)
    number_of_samples = get_number_of_samples(form_input)
    # samples_from_cruise = is_samples_from_cruise(form_input)
    # start_and_end_time = has_start_and_end_time(form_input)
    # number_of_locations = get_number_of_locations(form_input)
    # reference_properties = get_reference_properties(form_input)
    header = get_column_names(form_input)

    with open(template_internal_path, 'w') as csv_template:
        writer = csv.writer(csv_template)
        writer.writerow(header)
        writer.writerow(get_description_row(header))
        writer.writerow(get_options_row(header))
        # Write entity data to the lines
        for entity in samples:
            writer.writerow([
                entity.id,
                responsible_person_abbreviation
            ])
    return template_display_path


def main():
    parser = get_parser()
    args = parser.parse_args()
    configure(args)

    form_input = read_input(args)
    sample_entities = create_sample_entities(form_input)
    template_name = get_template_name(sample_entities)
    template_path = create_csv_template(template_name, sample_entities, form_input)
    logger.info("Your CSV-template has been prepared successfully.\n" +
                "Download the file <a href=/Shared/{}>here</a>.".format(template_path))
    try:
        send_mail_with_defaults(
            to=get_email_from_username(),
            subject=f"BIS sample registration {template_name}",
            body=create_email_with_link_text("sample registration template", template_path)
        )
    except KeyError as ke:
        logger.error(
            "There is a problem with the server's email configuration:\n\n"
            f"{ke}\n\nPlease contact your admin."
        )


if __name__ == "__main__":
    main()
